@@ -20,8 +20,9 @@
 #include <mutex>
 #include <string>
 #include "cubeb/cubeb.h"
+//#define ENABLE_NORMAL_LOG
+//#define ENABLE_VERBOSE_LOG
 #include "common.h"
-
 const uint32_t SAMPLE_FREQUENCY = 48000;
 const uint32_t TONE_FREQUENCY = 440;
 const double OUTPUT_AMPLITUDE = 0.25;
@@ -294,7 +295,7 @@ void run_loopback_duplex_test(bool is_float)
     cleanup_stream_at_exit(stream, cubeb_stream_destroy);
 
   cubeb_stream_start(stream);
-  delay(150);
+  delay(300);
   cubeb_stream_stop(stream);
 
   /* access after stop should not happen, but lock just in case and to appease sanitization tools */
@@ -348,6 +349,7 @@ void run_loopback_separate_streams_test(bool is_float)
   output_params.rate = SAMPLE_FREQUENCY;
   output_params.channels = 1;
   output_params.layout = CUBEB_LAYOUT_MONO;
+  output_params.prefs = CUBEB_STREAM_PREF_NONE;
 
   std::unique_ptr<user_state_loopback> user_data(new user_state_loopback());
   ASSERT_TRUE(!!user_data) << "Error allocating user data";
@@ -377,7 +379,7 @@ void run_loopback_separate_streams_test(bool is_float)
 
   cubeb_stream_start(input_stream);
   cubeb_stream_start(output_stream);
-  delay(150);
+  delay(300);
   cubeb_stream_stop(output_stream);
   cubeb_stream_stop(input_stream);
 
@@ -444,7 +446,7 @@ void run_loopback_silence_test(bool is_float)
     cleanup_input_stream_at_exit(input_stream, cubeb_stream_destroy);
 
   cubeb_stream_start(input_stream);
-  delay(50);
+  delay(300);
   cubeb_stream_stop(input_stream);
 
   /* access after stop should not happen, but lock just in case and to appease sanitization tools */
@@ -453,7 +455,7 @@ void run_loopback_silence_test(bool is_float)
 
   /* expect to have at least ~50ms of frames */
   ASSERT_GE(input_frames.size(), SAMPLE_FREQUENCY / 20);
-  double EPISILON = 0.000001;
+  double EPISILON = 0.0001;
   /* frames should be 0.0, but use epsilon to avoid possible issues with impls
   that may use ~0.0 silence values. */
   for (double frame : input_frames) {
@@ -515,6 +517,7 @@ void run_loopback_device_selection_test(bool is_float)
   output_params.rate = SAMPLE_FREQUENCY;
   output_params.channels = 1;
   output_params.layout = CUBEB_LAYOUT_MONO;
+  output_params.prefs = CUBEB_STREAM_PREF_NONE;
 
   std::unique_ptr<user_state_loopback> user_data(new user_state_loopback());
   ASSERT_TRUE(!!user_data) << "Error allocating user data";
@@ -544,7 +547,7 @@ void run_loopback_device_selection_test(bool is_float)
 
   cubeb_stream_start(input_stream);
   cubeb_stream_start(output_stream);
-  delay(150);
+  delay(300);
   cubeb_stream_stop(output_stream);
   cubeb_stream_stop(input_stream);
 
