@@ -8,8 +8,6 @@
 
 #include "logging.h"
 #include "nss.h"
-#include "ssl.h"
-#include "sslproto.h"
 
 #include "AudioSegment.h"
 #include "AudioStreamTrack.h"
@@ -24,9 +22,11 @@
 #include "transportflow.h"
 #include "transportlayerloopback.h"
 #include "transportlayerdtls.h"
+#include "transportlayersrtp.h"
 #include "mozilla/SyncRunnable.h"
 #include "mtransport_test_utils.h"
 #include "SharedBuffer.h"
+#include "MediaTransportHandler.h"
 
 #define GTEST_HAS_RTTI 0
 #include "gtest/gtest.h"
@@ -34,7 +34,7 @@
 using namespace mozilla;
 MOZ_MTLOG_MODULE("mediapipeline")
 
-static MtransportTestUtils *test_utils;
+static MtransportTestUtils* test_utils;
 
 namespace {
 
@@ -44,9 +44,10 @@ class FakeSourceMediaStream : public mozilla::SourceMediaStream {
 
   virtual ~FakeSourceMediaStream() override { mMainThreadDestroyed = true; }
 
-  virtual bool AppendToTrack(TrackID aID, MediaSegment *aSegment,
-                             MediaSegment *aRawSegment = nullptr) override {
-    return true;
+  virtual StreamTime AppendToTrack(
+      TrackID aID, MediaSegment* aSegment,
+      MediaSegment* aRawSegment = nullptr) override {
+    return aSegment->GetDuration();
   }
 };
 
@@ -68,7 +69,7 @@ class FakeMediaStreamTrackSource : public mozilla::dom::MediaStreamTrackSource {
 class FakeAudioStreamTrack : public mozilla::dom::AudioStreamTrack {
  public:
   FakeAudioStreamTrack()
-      : AudioStreamTrack(new DOMMediaStream(nullptr, nullptr), 0, 1,
+      : AudioStreamTrack(new DOMMediaStream(nullptr), 0, 1,
                          new FakeMediaStreamTrackSource()),
         mMutex("Fake AudioStreamTrack"),
         mStop(false),
@@ -86,25 +87,20 @@ class FakeAudioStreamTrack : public mozilla::dom::AudioStreamTrack {
     mTimer->Cancel();
   }
 
-  virtual void AddListener(MediaStreamTrackListener *aListener) override {
+  virtual void AddListener(MediaStreamTrackListener* aListener) override {
     mozilla::MutexAutoLock lock(mMutex);
     mListeners.push_back(aListener);
   }
 
-  virtual already_AddRefed<mozilla::dom::MediaStreamTrack> CloneInternal(
-      DOMMediaStream *aOwningStream, TrackID aTrackID) override {
-    return RefPtr<MediaStreamTrack>(new FakeAudioStreamTrack).forget();
-  }
-
  private:
-  std::vector<MediaStreamTrackListener *> mListeners;
+  std::vector<MediaStreamTrackListener*> mListeners;
   mozilla::Mutex mMutex;
   bool mStop;
   nsCOMPtr<nsITimer> mTimer;
   int mCount;
 
-  static void FakeAudioStreamTrackGenerateData(nsITimer *timer, void *closure) {
-    auto mst = static_cast<FakeAudioStreamTrack *>(closure);
+  static void FakeAudioStreamTrackGenerateData(nsITimer* timer, void* closure) {
+    auto mst = static_cast<FakeAudioStreamTrack*>(closure);
     const int AUDIO_BUFFER_SIZE = 1600;
     const int NUM_CHANNELS = 2;
 
@@ -115,7 +111,7 @@ class FakeAudioStreamTrack : public mozilla::dom::AudioStreamTrack {
 
     RefPtr<mozilla::SharedBuffer> samples = mozilla::SharedBuffer::Create(
         AUDIO_BUFFER_SIZE * NUM_CHANNELS * sizeof(int16_t));
-    int16_t *data = reinterpret_cast<int16_t *>(samples->Data());
+    int16_t* data = reinterpret_cast<int16_t*>(samples->Data());
     for (int i = 0; i < (AUDIO_BUFFER_SIZE * NUM_CHANNELS); i++) {
       // saw tooth audio sample
       data[i] = ((mst->mCount % 8) * 4000) - (7 * 4000) / 2;
@@ -123,132 +119,152 @@ class FakeAudioStreamTrack : public mozilla::dom::AudioStreamTrack {
     }
 
     mozilla::AudioSegment segment;
-    AutoTArray<const int16_t *, 1> channels;
+    AutoTArray<const int16_t*, 1> channels;
     channels.AppendElement(data);
     segment.AppendFrames(samples.forget(), channels, AUDIO_BUFFER_SIZE,
                          PRINCIPAL_HANDLE_NONE);
 
-    for (auto &listener : mst->mListeners) {
+    for (auto& listener : mst->mListeners) {
       listener->NotifyQueuedChanges(nullptr, 0, segment);
     }
   }
 };
 
-class TransportInfo {
+class LoopbackTransport : public MediaTransportHandler {
  public:
-  TransportInfo() : flow_(nullptr), loopback_(nullptr), dtls_(nullptr) {}
+  LoopbackTransport() : MediaTransportHandler(nullptr) {
+    SetState("mux", TransportLayer::TS_INIT, false);
+    SetState("mux", TransportLayer::TS_INIT, true);
+    SetState("non-mux", TransportLayer::TS_INIT, false);
+    SetState("non-mux", TransportLayer::TS_INIT, true);
+  }
 
-  static void InitAndConnect(TransportInfo &client, TransportInfo &server) {
-    client.Init(true);
-    server.Init(false);
-    client.PushLayers();
-    server.PushLayers();
+  static void InitAndConnect(LoopbackTransport& client,
+                             LoopbackTransport& server) {
     client.Connect(&server);
     server.Connect(&client);
   }
 
-  void Init(bool client) {
-    nsresult res;
+  void Connect(LoopbackTransport* peer) { peer_ = peer; }
 
-    flow_ = new TransportFlow();
-    loopback_ = new TransportLayerLoopback();
-    dtls_ = new TransportLayerDtls();
+  void Shutdown() { peer_ = nullptr; }
 
-    res = loopback_->Init();
-    if (res != NS_OK) {
-      FreeLayers();
+  RefPtr<IceLogPromise> GetIceLog(const nsCString& aPattern) override {
+    return nullptr;
+  }
+
+  void ClearIceLog() override {}
+  void EnterPrivateMode() override {}
+  void ExitPrivateMode() override {}
+
+  nsresult CreateIceCtx(const std::string& aName,
+                        const nsTArray<dom::RTCIceServer>& aIceServers,
+                        dom::RTCIceTransportPolicy aIcePolicy) override {
+    return NS_OK;
+  }
+
+  void Destroy() override {}
+
+  // We will probably be able to move the proxy lookup stuff into
+  // this class once we move mtransport to its own process.
+  void SetProxyServer(NrSocketProxyConfig&& aProxyConfig) override {}
+
+  void EnsureProvisionalTransport(const std::string& aTransportId,
+                                  const std::string& aLocalUfrag,
+                                  const std::string& aLocalPwd,
+                                  size_t aComponentCount) override {}
+
+  // We set default-route-only as late as possible because it depends on what
+  // capture permissions have been granted on the window, which could easily
+  // change between Init (ie; when the PC is created) and StartIceGathering
+  // (ie; when we set the local description).
+  void StartIceGathering(bool aDefaultRouteOnly,
+                         // TODO: It probably makes sense to look
+                         // this up internally
+                         const nsTArray<NrIceStunAddr>& aStunAddrs) override {}
+
+  void ActivateTransport(
+      const std::string& aTransportId, const std::string& aLocalUfrag,
+      const std::string& aLocalPwd, size_t aComponentCount,
+      const std::string& aUfrag, const std::string& aPassword,
+      const nsTArray<uint8_t>& aKeyDer, const nsTArray<uint8_t>& aCertDer,
+      SSLKEAType aAuthType, bool aDtlsClient, const DtlsDigestList& aDigests,
+      bool aPrivacyRequested) override {}
+
+  void RemoveTransportsExcept(
+      const std::set<std::string>& aTransportIds) override {}
+
+  void StartIceChecks(bool aIsControlling, bool aIsOfferer,
+                      const std::vector<std::string>& aIceOptions) override {}
+
+  void AddIceCandidate(const std::string& aTransportId,
+                       const std::string& aCandidate,
+                       const std::string& aUfrag) override {}
+
+  void UpdateNetworkState(bool aOnline) override {}
+
+  RefPtr<StatsPromise> GetIceStats(
+      const std::string& aTransportId, DOMHighResTimeStamp aNow,
+      std::unique_ptr<dom::RTCStatsReportInternal>&& aReport) override {
+    return nullptr;
+  }
+
+  void SendPacket(const std::string& aTransportId,
+                  MediaPacket&& aPacket) override {
+    peer_->SignalPacketReceived(aTransportId, aPacket);
+  }
+
+  void SetState(const std::string& aTransportId, TransportLayer::State aState,
+                bool aRtcp) {
+    if (aRtcp) {
+      MediaTransportHandler::OnRtcpStateChange(aTransportId, aState);
+    } else {
+      MediaTransportHandler::OnStateChange(aTransportId, aState);
     }
-    ASSERT_EQ((nsresult)NS_OK, res);
-
-    std::vector<uint16_t> ciphers;
-    ciphers.push_back(SRTP_AES128_CM_HMAC_SHA1_80);
-    dtls_->SetSrtpCiphers(ciphers);
-    dtls_->SetIdentity(DtlsIdentity::Generate());
-    dtls_->SetRole(client ? TransportLayerDtls::CLIENT
-                          : TransportLayerDtls::SERVER);
-    dtls_->SetVerificationAllowAll();
   }
 
-  void PushLayers() {
-    nsresult res;
-
-    nsAutoPtr<std::queue<TransportLayer *> > layers(
-        new std::queue<TransportLayer *>);
-    layers->push(loopback_);
-    layers->push(dtls_);
-    res = flow_->PushLayers(layers);
-    if (res != NS_OK) {
-      FreeLayers();
-    }
-    ASSERT_EQ((nsresult)NS_OK, res);
-  }
-
-  void Connect(TransportInfo *peer) {
-    MOZ_ASSERT(loopback_);
-    MOZ_ASSERT(peer->loopback_);
-
-    loopback_->Connect(peer->loopback_);
-  }
-
-  // Free the memory allocated at the beginning of Init
-  // if failure occurs before layers setup.
-  void FreeLayers() {
-    delete loopback_;
-    loopback_ = nullptr;
-    delete dtls_;
-    dtls_ = nullptr;
-  }
-
-  void Shutdown() {
-    if (loopback_) {
-      loopback_->Disconnect();
-    }
-    loopback_ = nullptr;
-    dtls_ = nullptr;
-    flow_ = nullptr;
-  }
-
-  RefPtr<TransportFlow> flow_;
-  TransportLayerLoopback *loopback_;
-  TransportLayerDtls *dtls_;
+ private:
+  RefPtr<MediaTransportHandler> peer_;
 };
 
 class TestAgent {
  public:
   TestAgent()
-      : audio_config_(109, "opus", 48000, 960, 2, 64000, false),
-        audio_conduit_(mozilla::AudioSessionConduit::Create()),
+      : audio_config_(109, "opus", 48000, 2, false),
+        audio_conduit_(mozilla::AudioSessionConduit::Create(
+            WebRtcCallWrapper::Create(), test_utils->sts_target())),
         audio_pipeline_(),
-        use_bundle_(false) {}
+        transport_(new LoopbackTransport) {}
 
-  static void ConnectRtp(TestAgent *client, TestAgent *server) {
-    TransportInfo::InitAndConnect(client->audio_rtp_transport_,
-                                  server->audio_rtp_transport_);
+  static void Connect(TestAgent* client, TestAgent* server) {
+    LoopbackTransport::InitAndConnect(*client->transport_, *server->transport_);
   }
 
-  static void ConnectRtcp(TestAgent *client, TestAgent *server) {
-    TransportInfo::InitAndConnect(client->audio_rtcp_transport_,
-                                  server->audio_rtcp_transport_);
+  virtual void CreatePipeline(const std::string& aTransportId) = 0;
+
+  void SetState(const std::string& aTransportId, TransportLayer::State aState,
+                bool aRtcp) {
+    mozilla::SyncRunnable::DispatchToThread(
+        test_utils->sts_target(),
+        WrapRunnable(transport_, &LoopbackTransport::SetState, aTransportId,
+                     aState, aRtcp));
   }
 
-  static void ConnectBundle(TestAgent *client, TestAgent *server) {
-    TransportInfo::InitAndConnect(client->bundle_transport_,
-                                  server->bundle_transport_);
+  void UpdateTransport(const std::string& aTransportId,
+                       nsAutoPtr<MediaPipelineFilter> aFilter) {
+    mozilla::SyncRunnable::DispatchToThread(
+        test_utils->sts_target(),
+        WrapRunnable(audio_pipeline_, &MediaPipeline::UpdateTransport_s,
+                     aTransportId, aFilter));
   }
 
-  virtual void CreatePipeline(bool aIsRtcpMux) = 0;
-
   void Stop() {
     MOZ_MTLOG(ML_DEBUG, "Stopping");
 
     if (audio_pipeline_) audio_pipeline_->Stop();
   }
 
-  void Shutdown_s() {
-    audio_rtp_transport_.Shutdown();
-    audio_rtcp_transport_.Shutdown();
-    bundle_transport_.Shutdown();
-  }
+  void Shutdown_s() { transport_->Shutdown(); }
 
   void Shutdown() {
     if (audio_pipeline_) audio_pipeline_->Shutdown_m();
@@ -282,8 +298,6 @@ class TestAgent {
     return audio_pipeline_->RtcpPacketsReceived();
   }
 
-  void SetUsingBundle(bool use_bundle) { use_bundle_ = use_bundle; }
-
  protected:
   mozilla::AudioCodecConfig audio_config_;
   RefPtr<mozilla::MediaSessionConduit> audio_conduit_;
@@ -292,48 +306,34 @@ class TestAgent {
   // both directions; only the sender's RTCP is sent, but the receiver should
   // be sending it too.
   RefPtr<mozilla::MediaPipeline> audio_pipeline_;
-  TransportInfo audio_rtp_transport_;
-  TransportInfo audio_rtcp_transport_;
-  TransportInfo bundle_transport_;
-  bool use_bundle_;
+  RefPtr<LoopbackTransport> transport_;
 };
 
 class TestAgentSend : public TestAgent {
  public:
   TestAgentSend() {
     mozilla::MediaConduitErrorCode err =
-        static_cast<mozilla::AudioSessionConduit *>(audio_conduit_.get())
+        static_cast<mozilla::AudioSessionConduit*>(audio_conduit_.get())
             ->ConfigureSendMediaCodec(&audio_config_);
     EXPECT_EQ(mozilla::kMediaConduitNoError, err);
 
     audio_stream_track_ = new FakeAudioStreamTrack();
   }
 
-  virtual void CreatePipeline(bool aIsRtcpMux) {
+  virtual void CreatePipeline(const std::string& aTransportId) {
     std::string test_pc;
 
-    if (aIsRtcpMux) {
-      ASSERT_FALSE(audio_rtcp_transport_.flow_);
-    }
-
     RefPtr<MediaPipelineTransmit> audio_pipeline =
-        new mozilla::MediaPipelineTransmit(
-            test_pc, nullptr, test_utils->sts_target(), false,
-            audio_stream_track_.get(), audio_conduit_);
+        new mozilla::MediaPipelineTransmit(test_pc, transport_, nullptr,
+                                           test_utils->sts_target(), false,
+                                           audio_conduit_);
 
+    audio_pipeline->SetTrack(audio_stream_track_.get());
     audio_pipeline->Start();
 
     audio_pipeline_ = audio_pipeline;
 
-    RefPtr<TransportFlow> rtp(audio_rtp_transport_.flow_);
-    RefPtr<TransportFlow> rtcp(audio_rtcp_transport_.flow_);
-
-    if (use_bundle_) {
-      rtp = bundle_transport_.flow_;
-      rtcp = nullptr;
-    }
-
-    audio_pipeline_->UpdateTransport_m(rtp, rtcp,
+    audio_pipeline_->UpdateTransport_m(aTransportId,
                                        nsAutoPtr<MediaPipelineFilter>(nullptr));
   }
 };
@@ -341,47 +341,35 @@ class TestAgentSend : public TestAgent {
 class TestAgentReceive : public TestAgent {
  public:
   TestAgentReceive() {
-    std::vector<mozilla::AudioCodecConfig *> codecs;
-    codecs.push_back(&audio_config_);
+    std::vector<UniquePtr<mozilla::AudioCodecConfig>> codecs;
+    codecs.emplace_back(new AudioCodecConfig(audio_config_));
 
     mozilla::MediaConduitErrorCode err =
-        static_cast<mozilla::AudioSessionConduit *>(audio_conduit_.get())
+        static_cast<mozilla::AudioSessionConduit*>(audio_conduit_.get())
             ->ConfigureRecvMediaCodecs(codecs);
     EXPECT_EQ(mozilla::kMediaConduitNoError, err);
   }
 
-  virtual void CreatePipeline(bool aIsRtcpMux) {
+  virtual void CreatePipeline(const std::string& aTransportId) {
     std::string test_pc;
 
-    if (aIsRtcpMux) {
-      ASSERT_FALSE(audio_rtcp_transport_.flow_);
-    }
-
     audio_pipeline_ = new mozilla::MediaPipelineReceiveAudio(
-        test_pc, nullptr, test_utils->sts_target(),
-        static_cast<mozilla::AudioSessionConduit *>(audio_conduit_.get()),
+        test_pc, transport_, nullptr, test_utils->sts_target(),
+        static_cast<mozilla::AudioSessionConduit*>(audio_conduit_.get()),
         nullptr);
 
     audio_pipeline_->Start();
 
-    RefPtr<TransportFlow> rtp(audio_rtp_transport_.flow_);
-    RefPtr<TransportFlow> rtcp(audio_rtcp_transport_.flow_);
-
-    if (use_bundle_) {
-      rtp = bundle_transport_.flow_;
-      rtcp = nullptr;
-    }
-
-    audio_pipeline_->UpdateTransport_m(rtp, rtcp, bundle_filter_);
+    audio_pipeline_->UpdateTransport_m(aTransportId, bundle_filter_);
   }
 
   void SetBundleFilter(nsAutoPtr<MediaPipelineFilter> filter) {
     bundle_filter_ = filter;
   }
 
-  void UpdateFilter_s(nsAutoPtr<MediaPipelineFilter> filter) {
-    audio_pipeline_->UpdateTransport_s(audio_rtp_transport_.flow_,
-                                       audio_rtcp_transport_.flow_, filter);
+  void UpdateTransport_s(const std::string& aTransportId,
+                         nsAutoPtr<MediaPipelineFilter> filter) {
+    audio_pipeline_->UpdateTransport_s(aTransportId, filter);
   }
 
  private:
@@ -402,24 +390,10 @@ class MediaPipelineTest : public ::testing::Test {
   }
 
   // Setup transport.
-  void InitTransports(bool aIsRtcpMux) {
-    // RTP, p1_ is server, p2_ is client
+  void InitTransports() {
     mozilla::SyncRunnable::DispatchToThread(
         test_utils->sts_target(),
-        WrapRunnableNM(&TestAgent::ConnectRtp, &p2_, &p1_));
-
-    // Create RTCP flows separately if we are not muxing them.
-    if (!aIsRtcpMux) {
-      // RTCP, p1_ is server, p2_ is client
-      mozilla::SyncRunnable::DispatchToThread(
-          test_utils->sts_target(),
-          WrapRunnableNM(&TestAgent::ConnectRtcp, &p2_, &p1_));
-    }
-
-    // BUNDLE, p1_ is server, p2_ is client
-    mozilla::SyncRunnable::DispatchToThread(
-        test_utils->sts_target(),
-        WrapRunnableNM(&TestAgent::ConnectBundle, &p2_, &p1_));
+        WrapRunnableNM(&TestAgent::Connect, &p2_, &p1_));
   }
 
   // Verify RTP and RTCP
@@ -438,10 +412,28 @@ class MediaPipelineTest : public ::testing::Test {
     p2_.SetBundleFilter(initialFilter);
 
     // Setup transport flows
-    InitTransports(aIsRtcpMux);
+    InitTransports();
 
-    p1_.CreatePipeline(aIsRtcpMux);
-    p2_.CreatePipeline(aIsRtcpMux);
+    std::string transportId = aIsRtcpMux ? "mux" : "non-mux";
+    p1_.CreatePipeline(transportId);
+    p2_.CreatePipeline(transportId);
+
+    // Set state of transports to CONNECTING. MediaPipeline doesn't really care
+    // about this transition, but we're trying to simluate what happens in a
+    // real case.
+    p1_.SetState(transportId, TransportLayer::TS_CONNECTING, false);
+    p1_.SetState(transportId, TransportLayer::TS_CONNECTING, true);
+    p2_.SetState(transportId, TransportLayer::TS_CONNECTING, false);
+    p2_.SetState(transportId, TransportLayer::TS_CONNECTING, true);
+
+    PR_Sleep(10);
+
+    // Set state of transports to OPEN (ie; connected). This should result in
+    // media flowing.
+    p1_.SetState(transportId, TransportLayer::TS_OPEN, false);
+    p1_.SetState(transportId, TransportLayer::TS_OPEN, true);
+    p2_.SetState(transportId, TransportLayer::TS_OPEN, false);
+    p2_.SetState(transportId, TransportLayer::TS_OPEN, true);
 
     if (bundle) {
       PR_Sleep(ms_until_filter_update);
@@ -454,9 +446,7 @@ class MediaPipelineTest : public ::testing::Test {
         refinedFilter->AddRemoteSSRC(p1_.GetLocalSSRC());
       }
 
-      mozilla::SyncRunnable::DispatchToThread(
-          test_utils->sts_target(),
-          WrapRunnable(&p2_, &TestAgentReceive::UpdateFilter_s, refinedFilter));
+      p2_.UpdateTransport(transportId, refinedFilter);
     }
 
     // wait for some RTP/RTCP tx and rx to happen
@@ -476,13 +466,13 @@ class MediaPipelineTest : public ::testing::Test {
       ASSERT_GE(p1_.GetAudioRtpCountSent(), 40);
       ASSERT_EQ(p1_.GetAudioRtpCountReceived(), p2_.GetAudioRtpCountSent());
       ASSERT_EQ(p1_.GetAudioRtpCountSent(), p2_.GetAudioRtpCountReceived());
-
-      // Calling ShutdownMedia_m on both pipelines does not stop the flow of
-      // RTCP. So, we might be off by one here.
-      ASSERT_LE(p2_.GetAudioRtcpCountReceived(), p1_.GetAudioRtcpCountSent());
-      ASSERT_GE(p2_.GetAudioRtcpCountReceived() + 1,
-                p1_.GetAudioRtcpCountSent());
     }
+
+    // No RTCP packets should have been dropped, because we do not filter them.
+    // Calling ShutdownMedia_m on both pipelines does not stop the flow of
+    // RTCP. So, we might be off by one here.
+    ASSERT_LE(p2_.GetAudioRtcpCountReceived(), p1_.GetAudioRtcpCountSent());
+    ASSERT_GE(p2_.GetAudioRtcpCountReceived() + 1, p1_.GetAudioRtcpCountSent());
   }
 
   void TestAudioReceiverBundle(
@@ -502,7 +492,7 @@ class MediaPipelineTest : public ::testing::Test {
 
 class MediaPipelineFilterTest : public ::testing::Test {
  public:
-  bool Filter(MediaPipelineFilter &filter, int32_t correlator, uint32_t ssrc,
+  bool Filter(MediaPipelineFilter& filter, int32_t correlator, uint32_t ssrc,
               uint8_t payload_type) {
     webrtc::RTPHeader header;
     header.ssrc = ssrc;
@@ -534,80 +524,6 @@ TEST_F(MediaPipelineFilterTest, TestSSRCFilter) {
 
 #define RTCP_TYPEINFO(num_rrs, type, size) 0x80 + num_rrs, type, 0, size
 
-const unsigned char rtcp_sr_s16[] = {
-    // zero rrs, size 6 words
-    RTCP_TYPEINFO(0, MediaPipelineFilter::SENDER_REPORT_T, 6),
-    REPORT_FRAGMENT(16)};
-
-const unsigned char rtcp_sr_s16_r17[] = {
-    // one rr, size 12 words
-    RTCP_TYPEINFO(1, MediaPipelineFilter::SENDER_REPORT_T, 12),
-    REPORT_FRAGMENT(16), REPORT_FRAGMENT(17)};
-
-const unsigned char unknown_type[] = {RTCP_TYPEINFO(1, 222, 0)};
-
-TEST_F(MediaPipelineFilterTest, TestEmptyFilterReport0) {
-  MediaPipelineFilter filter;
-  ASSERT_FALSE(filter.FilterSenderReport(rtcp_sr_s16, sizeof(rtcp_sr_s16)));
-}
-
-TEST_F(MediaPipelineFilterTest, TestFilterReport0) {
-  MediaPipelineFilter filter;
-  filter.AddRemoteSSRC(16);
-  ASSERT_TRUE(filter.FilterSenderReport(rtcp_sr_s16, sizeof(rtcp_sr_s16)));
-}
-
-TEST_F(MediaPipelineFilterTest, TestFilterReport0PTTruncated) {
-  MediaPipelineFilter filter;
-  filter.AddRemoteSSRC(16);
-  const unsigned char data[] = {0x80};
-  ASSERT_FALSE(filter.FilterSenderReport(data, sizeof(data)));
-}
-
-TEST_F(MediaPipelineFilterTest, TestFilterReport0CountTruncated) {
-  MediaPipelineFilter filter;
-  filter.AddRemoteSSRC(16);
-  const unsigned char *data = {};
-  ASSERT_FALSE(filter.FilterSenderReport(data, sizeof(data)));
-}
-
-TEST_F(MediaPipelineFilterTest, TestFilterReport1SSRCTruncated) {
-  MediaPipelineFilter filter;
-  filter.AddRemoteSSRC(16);
-  const unsigned char sr[] = {
-      RTCP_TYPEINFO(1, MediaPipelineFilter::SENDER_REPORT_T, 12),
-      REPORT_FRAGMENT(16), 0, 0, 0};
-  ASSERT_TRUE(filter.FilterSenderReport(sr, sizeof(sr)));
-}
-
-TEST_F(MediaPipelineFilterTest, TestFilterReport1BigSSRC) {
-  MediaPipelineFilter filter;
-  filter.AddRemoteSSRC(0x01020304);
-  const unsigned char sr[] = {
-      RTCP_TYPEINFO(1, MediaPipelineFilter::SENDER_REPORT_T, 12),
-      SSRC(0x01020304), REPORT_FRAGMENT(0x11121314)};
-  ASSERT_TRUE(filter.FilterSenderReport(sr, sizeof(sr)));
-}
-
-TEST_F(MediaPipelineFilterTest, TestFilterReportMatch) {
-  MediaPipelineFilter filter;
-  filter.AddRemoteSSRC(16);
-  ASSERT_TRUE(
-      filter.FilterSenderReport(rtcp_sr_s16_r17, sizeof(rtcp_sr_s16_r17)));
-}
-
-TEST_F(MediaPipelineFilterTest, TestFilterReportNoMatch) {
-  MediaPipelineFilter filter;
-  filter.AddRemoteSSRC(17);
-  ASSERT_FALSE(
-      filter.FilterSenderReport(rtcp_sr_s16_r17, sizeof(rtcp_sr_s16_r17)));
-}
-
-TEST_F(MediaPipelineFilterTest, TestFilterUnknownRTCPType) {
-  MediaPipelineFilter filter;
-  ASSERT_FALSE(filter.FilterSenderReport(unknown_type, sizeof(unknown_type)));
-}
-
 TEST_F(MediaPipelineFilterTest, TestCorrelatorFilter) {
   MediaPipelineFilter filter;
   filter.SetCorrelator(7777);
@@ -616,9 +532,6 @@ TEST_F(MediaPipelineFilterTest, TestCorrelatorFilter) {
   // This should also have resulted in the SSRC 16 being added to the filter
   ASSERT_TRUE(Filter(filter, 0, 16, 110));
   ASSERT_FALSE(Filter(filter, 0, 17, 110));
-
-  // rtcp_sr_s16 has 16 as an SSRC
-  ASSERT_TRUE(filter.FilterSenderReport(rtcp_sr_s16, sizeof(rtcp_sr_s16)));
 }
 
 TEST_F(MediaPipelineFilterTest, TestPayloadTypeFilter) {
@@ -628,15 +541,6 @@ TEST_F(MediaPipelineFilterTest, TestPayloadTypeFilter) {
   ASSERT_FALSE(Filter(filter, 0, 556, 111));
 }
 
-TEST_F(MediaPipelineFilterTest, TestPayloadTypeFilterSSRCUpdate) {
-  MediaPipelineFilter filter;
-  filter.AddUniquePT(110);
-  ASSERT_TRUE(Filter(filter, 0, 16, 110));
-
-  // rtcp_sr_s16 has 16 as an SSRC
-  ASSERT_TRUE(filter.FilterSenderReport(rtcp_sr_s16, sizeof(rtcp_sr_s16)));
-}
-
 TEST_F(MediaPipelineFilterTest, TestSSRCMovedWithCorrelator) {
   MediaPipelineFilter filter;
   filter.SetCorrelator(7777);
@@ -683,8 +587,6 @@ TEST_F(MediaPipelineTest, TestAudioSendBundle) {
   ASSERT_GT(p1_.GetAudioRtpCountSent(), p2_.GetAudioRtpCountReceived());
   ASSERT_GT(p2_.GetAudioRtpCountReceived(), 40);
   ASSERT_GT(p1_.GetAudioRtcpCountSent(), 1);
-  ASSERT_GT(p1_.GetAudioRtcpCountSent(), p2_.GetAudioRtcpCountReceived());
-  ASSERT_GT(p2_.GetAudioRtcpCountReceived(), 0);
 }
 
 TEST_F(MediaPipelineTest, TestAudioSendEmptyBundleFilter) {
