@@ -5,11 +5,16 @@
 const DB_VERSION = 6; // The database schema version
 const PERMISSION_SAVE_LOGINS = "login-saving";
 
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
+const { XPCOMUtils } = ChromeUtils.import(
+  "resource://gre/modules/XPCOMUtils.jsm"
+);
+const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
 
-ChromeUtils.defineModuleGetter(this, "LoginHelper",
-                               "resource://gre/modules/LoginHelper.jsm");
+ChromeUtils.defineModuleGetter(
+  this,
+  "LoginHelper",
+  "resource://gre/modules/LoginHelper.jsm"
+);
 
 /**
  * Object that manages a database transaction properly so consumers don't have
@@ -25,29 +30,38 @@ function Transaction(aDatabase) {
   try {
     this._db.beginTransaction();
     this._hasTransaction = true;
-  } catch (e) { /* om nom nom exceptions */ }
+  } catch (e) {
+    /* om nom nom exceptions */
+  }
 }
 
 Transaction.prototype = {
   commit() {
-    if (this._hasTransaction)
+    if (this._hasTransaction) {
       this._db.commitTransaction();
+    }
   },
 
   rollback() {
-    if (this._hasTransaction)
+    if (this._hasTransaction) {
       this._db.rollbackTransaction();
+    }
   },
 };
 
-
-function LoginManagerStorage_mozStorage() { }
+function LoginManagerStorage_mozStorage() {}
 
 LoginManagerStorage_mozStorage.prototype = {
-
   classID: Components.ID("{8c2023b9-175c-477e-9761-44ae7b549756}"),
-  QueryInterface: XPCOMUtils.generateQI([Ci.nsILoginManagerStorage,
-                                          Ci.nsIInterfaceRequestor]),
+  QueryInterface: ChromeUtils.generateQI([
+    Ci.nsILoginManagerStorage,
+    Ci.nsIInterfaceRequestor,
+  ]),
+
+  _xpcom_factory: XPCOMUtils.generateSingletonFactory(
+    this.LoginManagerStorage_mozStorage
+  ),
+
   getInterface(aIID) {
     if (aIID.equals(Ci.nsIVariant)) {
       // Allows unwrapping the JavaScript object for regression tests.
@@ -58,101 +72,110 @@ LoginManagerStorage_mozStorage.prototype = {
       return this._dbConnection;
     }
 
-    throw new Components.Exception("Interface not available", Cr.NS_ERROR_NO_INTERFACE);
+    throw new Components.Exception(
+      "Interface not available",
+      Cr.NS_ERROR_NO_INTERFACE
+    );
   },
 
-  __crypto: null,  // nsILoginManagerCrypto service
+  __crypto: null, // nsILoginManagerCrypto service
   get _crypto() {
-    if (!this.__crypto)
-      this.__crypto = Cc["@mozilla.org/login-manager/crypto/SDR;1"].
-                      getService(Ci.nsILoginManagerCrypto);
+    if (!this.__crypto) {
+      this.__crypto = Cc["@mozilla.org/login-manager/crypto/SDR;1"].getService(
+        Ci.nsILoginManagerCrypto
+      );
+    }
     return this.__crypto;
   },
 
-  __profileDir: null,  // nsIFile for the user's profile dir
+  __profileDir: null, // nsIFile for the user's profile dir
   get _profileDir() {
-    if (!this.__profileDir)
+    if (!this.__profileDir) {
       this.__profileDir = Services.dirsvc.get("ProfD", Ci.nsIFile);
+    }
     return this.__profileDir;
   },
 
   __uuidService: null,
   get _uuidService() {
-    if (!this.__uuidService)
-      this.__uuidService = Cc["@mozilla.org/uuid-generator;1"].
-                           getService(Ci.nsIUUIDGenerator);
+    if (!this.__uuidService) {
+      this.__uuidService = Cc["@mozilla.org/uuid-generator;1"].getService(
+        Ci.nsIUUIDGenerator
+      );
+    }
     return this.__uuidService;
   },
 
-
   // The current database schema.
   _dbSchema: {
     tables: {
-      moz_logins:         "id                  INTEGER PRIMARY KEY," +
-                          "hostname            TEXT NOT NULL," +
-                          "httpRealm           TEXT," +
-                          "formSubmitURL       TEXT," +
-                          "usernameField       TEXT NOT NULL," +
-                          "passwordField       TEXT NOT NULL," +
-                          "encryptedUsername   TEXT NOT NULL," +
-                          "encryptedPassword   TEXT NOT NULL," +
-                          "guid                TEXT," +
-                          "encType             INTEGER," +
-                          "timeCreated         INTEGER," +
-                          "timeLastUsed        INTEGER," +
-                          "timePasswordChanged INTEGER," +
-                          "timesUsed           INTEGER",
+      moz_logins:
+        "id                  INTEGER PRIMARY KEY," +
+        "hostname            TEXT NOT NULL," +
+        "httpRealm           TEXT," +
+        "formSubmitURL       TEXT," +
+        "usernameField       TEXT NOT NULL," +
+        "passwordField       TEXT NOT NULL," +
+        "encryptedUsername   TEXT NOT NULL," +
+        "encryptedPassword   TEXT NOT NULL," +
+        "guid                TEXT," +
+        "encType             INTEGER," +
+        "timeCreated         INTEGER," +
+        "timeLastUsed        INTEGER," +
+        "timePasswordChanged INTEGER," +
+        "timesUsed           INTEGER",
       // Changes must be reflected in this._dbAreExpectedColumnsPresent(),
       // this._searchLogins(), and this.modifyLogin().
 
-      moz_disabledHosts:  "id                 INTEGER PRIMARY KEY," +
-                          "hostname           TEXT UNIQUE ON CONFLICT REPLACE",
+      moz_disabledHosts:
+        "id                 INTEGER PRIMARY KEY," +
+        "hostname           TEXT UNIQUE ON CONFLICT REPLACE",
 
-      moz_deleted_logins: "id                  INTEGER PRIMARY KEY," +
-                          "guid                TEXT," +
-                          "timeDeleted         INTEGER",
+      moz_deleted_logins:
+        "id                  INTEGER PRIMARY KEY," +
+        "guid                TEXT," +
+        "timeDeleted         INTEGER",
     },
     indices: {
       moz_logins_hostname_index: {
         table: "moz_logins",
-        columns: ["hostname"]
+        columns: ["hostname"],
       },
       moz_logins_hostname_formSubmitURL_index: {
         table: "moz_logins",
-        columns: ["hostname", "formSubmitURL"]
+        columns: ["hostname", "formSubmitURL"],
       },
       moz_logins_hostname_httpRealm_index: {
-          table: "moz_logins",
-          columns: ["hostname", "httpRealm"]
+        table: "moz_logins",
+        columns: ["hostname", "httpRealm"],
       },
       moz_logins_guid_index: {
-          table: "moz_logins",
-          columns: ["guid"]
+        table: "moz_logins",
+        columns: ["guid"],
       },
       moz_logins_encType_index: {
-          table: "moz_logins",
-          columns: ["encType"]
-      }
-    }
+        table: "moz_logins",
+        columns: ["encType"],
+      },
+    },
   },
-  _dbConnection: null,  // The database connection
-  _dbStmts: null,  // Database statements for memoization
-
-  _signonsFile: null,  // nsIFile for "signons.sqlite"
+  _dbConnection: null, // The database connection
+  _dbStmts: null, // Database statements for memoization
 
+  _signonsFile: null, // nsIFile for "signons.sqlite"
 
   /*
    * Internal method used by regression tests only.  It overrides the default
    * database location.
    */
   initWithFile(aDBFile) {
-    if (aDBFile)
+    if (aDBFile) {
       this._signonsFile = aDBFile;
+    }
 
     this.initialize();
   },
 
-
   initialize() {
     this._dbStmts = {};
 
@@ -179,13 +202,13 @@ LoginManagerStorage_mozStorage.prototype = {
     } catch (e) {
       this.log("Initialization failed: " + e);
       // If the import fails on first run, we want to delete the db
-      if (isFirstRun && e == "Import failed")
+      if (isFirstRun && e == "Import failed") {
         this._dbCleanup(false);
+      }
       throw new Error("Initialization failed");
     }
   },
 
-
   /**
    * Internal method used by regression tests only.  It is called before
    * replacing this storage module with a new instance.
@@ -194,14 +217,13 @@ LoginManagerStorage_mozStorage.prototype = {
     return Promise.resolve();
   },
 
-
   addLogin(login, preEncrypted = false) {
     // Throws if there are bogus values.
     LoginHelper.checkLoginValues(login);
 
-    let [encUsername, encPassword, encType] = preEncrypted ?
-      [login.username, login.password, this._crypto.defaultEncType] :
-      this._encryptLogin(login);
+    let [encUsername, encPassword, encType] = preEncrypted
+      ? [login.username, login.password, this._crypto.defaultEncType]
+      : this._encryptLogin(login);
 
     // Clone the login, so we don't modify the caller's object.
     let loginClone = login.clone();
@@ -209,48 +231,53 @@ LoginManagerStorage_mozStorage.prototype = {
     // Initialize the nsILoginMetaInfo fields, unless the caller gave us values
     loginClone.QueryInterface(Ci.nsILoginMetaInfo);
     if (loginClone.guid) {
-      if (!this._isGuidUnique(loginClone.guid))
+      if (!this._isGuidUnique(loginClone.guid)) {
         throw new Error("specified GUID already exists");
+      }
     } else {
       loginClone.guid = this._uuidService.generateUUID().toString();
     }
 
     // Set timestamps
     let currentTime = Date.now();
-    if (!loginClone.timeCreated)
+    if (!loginClone.timeCreated) {
       loginClone.timeCreated = currentTime;
-    if (!loginClone.timeLastUsed)
+    }
+    if (!loginClone.timeLastUsed) {
       loginClone.timeLastUsed = currentTime;
-    if (!loginClone.timePasswordChanged)
+    }
+    if (!loginClone.timePasswordChanged) {
       loginClone.timePasswordChanged = currentTime;
-    if (!loginClone.timesUsed)
+    }
+    if (!loginClone.timesUsed) {
       loginClone.timesUsed = 1;
+    }
 
     let query =
-        "INSERT INTO moz_logins " +
-        "(hostname, httpRealm, formSubmitURL, usernameField, " +
-         "passwordField, encryptedUsername, encryptedPassword, " +
-         "guid, encType, timeCreated, timeLastUsed, timePasswordChanged, " +
-         "timesUsed) " +
-        "VALUES (:hostname, :httpRealm, :formSubmitURL, :usernameField, " +
-                ":passwordField, :encryptedUsername, :encryptedPassword, " +
-                ":guid, :encType, :timeCreated, :timeLastUsed, " +
-                ":timePasswordChanged, :timesUsed)";
+      "INSERT INTO moz_logins " +
+      "(hostname, httpRealm, formSubmitURL, usernameField, " +
+      "passwordField, encryptedUsername, encryptedPassword, " +
+      "guid, encType, timeCreated, timeLastUsed, timePasswordChanged, " +
+      "timesUsed) " +
+      "VALUES (:hostname, :httpRealm, :formSubmitURL, :usernameField, " +
+      ":passwordField, :encryptedUsername, :encryptedPassword, " +
+      ":guid, :encType, :timeCreated, :timeLastUsed, " +
+      ":timePasswordChanged, :timesUsed)";
 
     let params = {
-      hostname:            loginClone.hostname,
-      httpRealm:           loginClone.httpRealm,
-      formSubmitURL:       loginClone.formSubmitURL,
-      usernameField:       loginClone.usernameField,
-      passwordField:       loginClone.passwordField,
-      encryptedUsername:   encUsername,
-      encryptedPassword:   encPassword,
-      guid:                loginClone.guid,
+      hostname: loginClone.hostname,
+      httpRealm: loginClone.httpRealm,
+      formSubmitURL: loginClone.formSubmitURL,
+      usernameField: loginClone.usernameField,
+      passwordField: loginClone.passwordField,
+      encryptedUsername: encUsername,
+      encryptedPassword: encPassword,
+      guid: loginClone.guid,
       encType,
-      timeCreated:         loginClone.timeCreated,
-      timeLastUsed:        loginClone.timeLastUsed,
+      timeCreated: loginClone.timeCreated,
+      timeLastUsed: loginClone.timeLastUsed,
       timePasswordChanged: loginClone.timePasswordChanged,
-      timesUsed:           loginClone.timesUsed
+      timesUsed: loginClone.timesUsed,
     };
 
     let stmt;
@@ -271,14 +298,14 @@ LoginManagerStorage_mozStorage.prototype = {
     return loginClone;
   },
 
-
   removeLogin(login) {
     let [idToDelete, storedLogin] = this._getIdForLogin(login);
-    if (!idToDelete)
+    if (!idToDelete) {
       throw new Error("No matching logins");
+    }
 
     // Execute the statement & remove from DB
-    let query  = "DELETE FROM moz_logins WHERE id = :id";
+    let query = "DELETE FROM moz_logins WHERE id = :id";
     let params = { id: idToDelete };
     let stmt;
     let transaction = new Transaction(this._dbConnection);
@@ -301,62 +328,68 @@ LoginManagerStorage_mozStorage.prototype = {
 
   modifyLogin(oldLogin, newLoginData) {
     let [idToModify, oldStoredLogin] = this._getIdForLogin(oldLogin);
-    if (!idToModify)
+    if (!idToModify) {
       throw new Error("No matching logins");
+    }
 
     let newLogin = LoginHelper.buildModifiedLogin(oldStoredLogin, newLoginData);
 
     // Check if the new GUID is duplicate.
-    if (newLogin.guid != oldStoredLogin.guid &&
-        !this._isGuidUnique(newLogin.guid)) {
+    if (
+      newLogin.guid != oldStoredLogin.guid &&
+      !this._isGuidUnique(newLogin.guid)
+    ) {
       throw new Error("specified GUID already exists");
     }
 
     // Look for an existing entry in case key properties changed.
     if (!newLogin.matches(oldLogin, true)) {
-      let logins = this.findLogins({}, newLogin.hostname,
-                                   newLogin.formSubmitURL,
-                                   newLogin.httpRealm);
+      let logins = this.findLogins(
+        newLogin.hostname,
+        newLogin.formSubmitURL,
+        newLogin.httpRealm
+      );
 
-      if (logins.some(login => newLogin.matches(login, true)))
+      if (logins.some(login => newLogin.matches(login, true))) {
         throw new Error("This login already exists.");
+      }
     }
 
     // Get the encrypted value of the username and password.
     let [encUsername, encPassword, encType] = this._encryptLogin(newLogin);
 
     let query =
-        "UPDATE moz_logins " +
-        "SET hostname = :hostname, " +
-            "httpRealm = :httpRealm, " +
-            "formSubmitURL = :formSubmitURL, " +
-            "usernameField = :usernameField, " +
-            "passwordField = :passwordField, " +
-            "encryptedUsername = :encryptedUsername, " +
-            "encryptedPassword = :encryptedPassword, " +
-            "guid = :guid, " +
-            "encType = :encType, " +
-            "timeCreated = :timeCreated, " +
-            "timeLastUsed = :timeLastUsed, " +
-            "timePasswordChanged = :timePasswordChanged, " +
-            "timesUsed = :timesUsed " +
-        "WHERE id = :id";
+      "UPDATE moz_logins " +
+      "SET hostname = :hostname, " +
+      "httpRealm = :httpRealm, " +
+      "formSubmitURL = :formSubmitURL, " +
+      "usernameField = :usernameField, " +
+      "passwordField = :passwordField, " +
+      "encryptedUsername = :encryptedUsername, " +
+      "encryptedPassword = :encryptedPassword, " +
+      "guid = :guid, " +
+      "encType = :encType, " +
+      "timeCreated = :timeCreated, " +
+      "timeLastUsed = :timeLastUsed, " +
+      "timePasswordChanged = :timePasswordChanged, " +
+      "timesUsed = :timesUsed " +
+      "WHERE id = :id";
 
     let params = {
-      id:                  idToModify,
-      hostname:            newLogin.hostname,
-      httpRealm:           newLogin.httpRealm,
-      formSubmitURL:       newLogin.formSubmitURL,
-      usernameField:       newLogin.usernameField,
-      passwordField:       newLogin.passwordField,
-      encryptedUsername:   encUsername,
-      encryptedPassword:   encPassword,
-      guid:                newLogin.guid,
+      id: idToModify,
+      hostname: newLogin.hostname,
+      httpRealm: newLogin.httpRealm,
+      formSubmitURL: newLogin.formSubmitURL,
+      usernameField: newLogin.usernameField,
+      passwordField: newLogin.passwordField,
+      encryptedUsername: encUsername,
+      encryptedPassword: encPassword,
+      guid: newLogin.guid,
       encType,
-      timeCreated:         newLogin.timeCreated,
-      timeLastUsed:        newLogin.timeLastUsed,
+      timeCreated: newLogin.timeCreated,
+      timeLastUsed: newLogin.timeLastUsed,
       timePasswordChanged: newLogin.timePasswordChanged,
-      timesUsed:           newLogin.timesUsed
+      timesUsed: newLogin.timesUsed,
     };
 
     let stmt;
@@ -375,36 +408,30 @@ LoginManagerStorage_mozStorage.prototype = {
     LoginHelper.notifyStorageChanged("modifyLogin", [oldStoredLogin, newLogin]);
   },
 
-
   /**
    * Returns an array of nsILoginInfo.
    */
-  getAllLogins(count) {
+  getAllLogins() {
     let [logins, ids] = this._searchLogins({});
 
     // decrypt entries for caller.
     logins = this._decryptLogins(logins);
 
     this.log("_getAllLogins: returning " + logins.length + " logins.");
-    if (count)
-      count.value = logins.length; // needed for XPCOM
     return logins;
   },
 
-
   /**
    * Public wrapper around _searchLogins to convert the nsIPropertyBag to a
    * JavaScript object and decrypt the results.
    *
    * @return {nsILoginInfo[]} which are decrypted.
    */
-  searchLogins(count, matchData) {
+  searchLogins(matchData) {
     let realMatchData = {};
     let options = {};
     // Convert nsIPropertyBag to normal JS object
-    let propEnum = matchData.enumerator;
-    while (propEnum.hasMoreElements()) {
-      let prop = propEnum.getNext().QueryInterface(Ci.nsIProperty);
+    for (let prop of matchData.enumerator) {
       switch (prop.name) {
         // Some property names aren't field names but are special options to affect the search.
         case "schemeUpgrades": {
@@ -423,11 +450,9 @@ LoginManagerStorage_mozStorage.prototype = {
     // Decrypt entries found for the caller.
     logins = this._decryptLogins(logins);
 
-    count.value = logins.length; // needed for XPCOM
     return logins;
   },
 
-
   /**
    * Private method to perform arbitrary searches on any field. Decryption is
    * left to the caller.
@@ -436,10 +461,14 @@ LoginManagerStorage_mozStorage.prototype = {
    * is an array of encrypted nsLoginInfo and ids is an array of associated
    * ids in the database.
    */
-  _searchLogins(matchData, aOptions = {
-    schemeUpgrades: false,
-  }) {
-    let conditions = [], params = {};
+  _searchLogins(
+    matchData,
+    aOptions = {
+      schemeUpgrades: false,
+    }
+  ) {
+    let conditions = [],
+      params = {};
 
     for (let field in matchData) {
       let value = matchData[field];
@@ -450,15 +479,18 @@ LoginManagerStorage_mozStorage.prototype = {
             // Historical compatibility requires this special case
             condition = "formSubmitURL = '' OR ";
           }
-          // Fall through
+        // Fall through
         case "hostname":
           if (value != null) {
             condition += `${field} = :${field}`;
             params[field] = value;
             let valueURI;
             try {
-              if (aOptions.schemeUpgrades && (valueURI = Services.io.newURI(value)) &&
-                  valueURI.scheme == "https") {
+              if (
+                aOptions.schemeUpgrades &&
+                (valueURI = Services.io.newURI(value)) &&
+                valueURI.scheme == "https"
+              ) {
                 condition += ` OR ${field} = :http${field}`;
                 params["http" + field] = "http://" + valueURI.displayHostPort;
               }
@@ -468,7 +500,7 @@ LoginManagerStorage_mozStorage.prototype = {
             }
             break;
           }
-          // Fall through
+        // Fall through
         // Normal cases.
         case "httpRealm":
         case "id":
@@ -506,18 +538,25 @@ LoginManagerStorage_mozStorage.prototype = {
     }
 
     let stmt;
-    let logins = [], ids = [];
+    let logins = [],
+      ids = [];
     try {
       stmt = this._dbCreateStatement(query, params);
       // We can't execute as usual here, since we're iterating over rows
       while (stmt.executeStep()) {
         // Create the new nsLoginInfo object, push to array
-        let login = Cc["@mozilla.org/login-manager/loginInfo;1"].
-                    createInstance(Ci.nsILoginInfo);
-        login.init(stmt.row.hostname, stmt.row.formSubmitURL,
-                   stmt.row.httpRealm, stmt.row.encryptedUsername,
-                   stmt.row.encryptedPassword, stmt.row.usernameField,
-                   stmt.row.passwordField);
+        let login = Cc["@mozilla.org/login-manager/loginInfo;1"].createInstance(
+          Ci.nsILoginInfo
+        );
+        login.init(
+          stmt.row.hostname,
+          stmt.row.formSubmitURL,
+          stmt.row.httpRealm,
+          stmt.row.encryptedUsername,
+          stmt.row.encryptedPassword,
+          stmt.row.usernameField,
+          stmt.row.passwordField
+        );
         // set nsILoginMetaInfo values
         login.QueryInterface(Ci.nsILoginMetaInfo);
         login.guid = stmt.row.guid;
@@ -547,20 +586,20 @@ LoginManagerStorage_mozStorage.prototype = {
     let stmt = null;
     try {
       this.log("Storing " + aLogin.guid + " in deleted passwords\n");
-      let query = "INSERT INTO moz_deleted_logins (guid, timeDeleted) VALUES (:guid, :timeDeleted)";
-      let params = { guid: aLogin.guid,
-                     timeDeleted: Date.now() };
+      let query =
+        "INSERT INTO moz_deleted_logins (guid, timeDeleted) VALUES (:guid, :timeDeleted)";
+      let params = { guid: aLogin.guid, timeDeleted: Date.now() };
       let stmt = this._dbCreateStatement(query, params);
       stmt.execute();
     } catch (ex) {
       throw ex;
     } finally {
-      if (stmt)
+      if (stmt) {
         stmt.reset();
+      }
     }
   },
 
-
   /**
    * Removes all logins from storage.
    */
@@ -591,34 +630,35 @@ LoginManagerStorage_mozStorage.prototype = {
     LoginHelper.notifyStorageChanged("removeAllLogins", null);
   },
 
-
-  findLogins(count, hostname, formSubmitURL, httpRealm) {
+  findLogins(hostname, formSubmitURL, httpRealm) {
     let loginData = {
       hostname,
       formSubmitURL,
-      httpRealm
+      httpRealm,
     };
-    let matchData = { };
-    for (let field of ["hostname", "formSubmitURL", "httpRealm"])
-      if (loginData[field] != "")
+    let matchData = {};
+    for (let field of ["hostname", "formSubmitURL", "httpRealm"]) {
+      if (loginData[field] != "") {
         matchData[field] = loginData[field];
+      }
+    }
     let [logins, ids] = this._searchLogins(matchData);
 
     // Decrypt entries found for the caller.
     logins = this._decryptLogins(logins);
 
     this.log("_findLogins: returning " + logins.length + " logins");
-    count.value = logins.length; // needed for XPCOM
     return logins;
   },
 
-
   countLogins(hostname, formSubmitURL, httpRealm) {
-
     let _countLoginsHelper = (hostname, formSubmitURL, httpRealm) => {
       // Do checks for null and empty strings, adjust conditions and params
-      let [conditions, params] =
-          this._buildConditionsAndParams(hostname, formSubmitURL, httpRealm);
+      let [conditions, params] = this._buildConditionsAndParams(
+        hostname,
+        formSubmitURL,
+        httpRealm
+      );
 
       let query = "SELECT COUNT(1) AS numLogins FROM moz_logins";
       if (conditions.length) {
@@ -646,27 +686,26 @@ LoginManagerStorage_mozStorage.prototype = {
     return resultLogins;
   },
 
-
   get uiBusy() {
     return this._crypto.uiBusy;
   },
 
-
   get isLoggedIn() {
     return this._crypto.isLoggedIn;
   },
 
-
   /**
    * Returns an array with two items: [id, login]. If the login was not
    * found, both items will be null. The returned login contains the actual
    * stored login (useful for looking at the actual nsILoginMetaInfo values).
    */
   _getIdForLogin(login) {
-    let matchData = { };
-    for (let field of ["hostname", "formSubmitURL", "httpRealm"])
-      if (login[field] != "")
+    let matchData = {};
+    for (let field of ["hostname", "formSubmitURL", "httpRealm"]) {
+      if (login[field] != "") {
         matchData[field] = login[field];
+      }
+    }
     let [logins, ids] = this._searchLogins(matchData);
 
     let id = null;
@@ -679,8 +718,9 @@ LoginManagerStorage_mozStorage.prototype = {
     for (let i = 0; i < logins.length; i++) {
       let [decryptedLogin] = this._decryptLogins([logins[i]]);
 
-      if (!decryptedLogin || !decryptedLogin.equals(login))
+      if (!decryptedLogin || !decryptedLogin.equals(login)) {
         continue;
+      }
 
       // We've found a match, set id and break
       foundLogin = decryptedLogin;
@@ -691,14 +731,14 @@ LoginManagerStorage_mozStorage.prototype = {
     return [id, foundLogin];
   },
 
-
   /**
    * Adjusts the WHERE conditions and parameters for statements prior to the
    * statement being created. This fixes the cases where nulls are involved
    * and the empty string is supposed to be a wildcard match
    */
   _buildConditionsAndParams(hostname, formSubmitURL, httpRealm) {
-    let conditions = [], params = {};
+    let conditions = [],
+      params = {};
 
     if (hostname == null) {
       conditions.push("hostname isnull");
@@ -724,12 +764,12 @@ LoginManagerStorage_mozStorage.prototype = {
     return [conditions, params];
   },
 
-
   /**
    * Checks to see if the specified GUID already exists.
    */
   _isGuidUnique(guid) {
-    let query = "SELECT COUNT(1) AS numLogins FROM moz_logins WHERE guid = :guid";
+    let query =
+      "SELECT COUNT(1) AS numLogins FROM moz_logins WHERE guid = :guid";
     let params = { guid };
 
     let stmt, numLogins;
@@ -745,10 +785,9 @@ LoginManagerStorage_mozStorage.prototype = {
       }
     }
 
-    return (numLogins == 0);
+    return numLogins == 0;
   },
 
-
   /**
    * Returns the encrypted username, password, and encrypton type for the specified
    * login. Can throw if the user cancels a master password entry.
@@ -756,12 +795,11 @@ LoginManagerStorage_mozStorage.prototype = {
   _encryptLogin(login) {
     let encUsername = this._crypto.encrypt(login.username);
     let encPassword = this._crypto.encrypt(login.password);
-    let encType     = this._crypto.defaultEncType;
+    let encType = this._crypto.defaultEncType;
 
     return [encUsername, encPassword, encType];
   },
 
-
   /**
    * Decrypts username and password fields in the provided array of
    * logins.
@@ -783,8 +821,9 @@ LoginManagerStorage_mozStorage.prototype = {
       } catch (e) {
         // If decryption failed (corrupt entry?), just skip it.
         // Rethrow other errors (like canceling entry of a master pw)
-        if (e.result == Cr.NS_ERROR_FAILURE)
+        if (e.result == Cr.NS_ERROR_FAILURE) {
           continue;
+        }
         throw e;
       }
       result.push(login);
@@ -793,7 +832,6 @@ LoginManagerStorage_mozStorage.prototype = {
     return result;
   },
 
-
   // Database Creation & Access
 
   /**
@@ -810,13 +848,14 @@ LoginManagerStorage_mozStorage.prototype = {
       this._dbStmts[query] = wrappedStmt;
     }
     // Replace parameters, must be done 1 at a time
-    if (params)
-      for (let i in params)
+    if (params) {
+      for (let i in params) {
         wrappedStmt.params[i] = params[i];
+      }
+    }
     return wrappedStmt;
   },
 
-
   /**
    * Attempts to initialize the database. This creates the file if it doesn't
    * exist, performs any migrations, etc. Return if this is the first run.
@@ -853,7 +892,7 @@ LoginManagerStorage_mozStorage.prototype = {
       case "profile-before-change":
         Services.obs.removeObserver(this, "profile-before-change");
         this._dbClose();
-      break;
+        break;
     }
   },
 
@@ -863,31 +902,34 @@ LoginManagerStorage_mozStorage.prototype = {
     this._dbConnection.schemaVersion = DB_VERSION;
   },
 
-
   _dbCreateSchema() {
     this._dbCreateTables();
     this._dbCreateIndices();
   },
 
-
   _dbCreateTables() {
     this.log("Creating Tables");
-    for (let name in this._dbSchema.tables)
+    for (let name in this._dbSchema.tables) {
       this._dbConnection.createTable(name, this._dbSchema.tables[name]);
+    }
   },
 
-
   _dbCreateIndices() {
     this.log("Creating Indices");
     for (let name in this._dbSchema.indices) {
       let index = this._dbSchema.indices[name];
-      let statement = "CREATE INDEX IF NOT EXISTS " + name + " ON " + index.table +
-                      "(" + index.columns.join(", ") + ")";
+      let statement =
+        "CREATE INDEX IF NOT EXISTS " +
+        name +
+        " ON " +
+        index.table +
+        "(" +
+        index.columns.join(", ") +
+        ")";
       this._dbConnection.executeSimpleSQL(statement);
     }
   },
 
-
   _dbMigrate(oldVersion) {
     this.log("Attempting to migrate from version " + oldVersion);
 
@@ -899,9 +941,12 @@ LoginManagerStorage_mozStorage.prototype = {
       // the DB and start from scratch. [Future incompatible upgrades
       // should swtich to a different table or file.]
 
-      if (!this._dbAreExpectedColumnsPresent())
-        throw Components.Exception("DB is missing expected columns",
-                                   Cr.NS_ERROR_FILE_CORRUPTED);
+      if (!this._dbAreExpectedColumnsPresent()) {
+        throw Components.Exception(
+          "DB is missing expected columns",
+          Cr.NS_ERROR_FILE_CORRUPTED
+        );
+      }
 
       // Change the stored version to the current version. If the user
       // runs the newer code again, it will see the lower version number
@@ -931,7 +976,6 @@ LoginManagerStorage_mozStorage.prototype = {
     this.log("DB migration completed.");
   },
 
-
   /**
    * Version 2 adds a GUID column. Existing logins are assigned a random GUID.
    */
@@ -942,7 +986,8 @@ LoginManagerStorage_mozStorage.prototype = {
       query = "ALTER TABLE moz_logins ADD COLUMN guid TEXT";
       this._dbConnection.executeSimpleSQL(query);
 
-      query = "CREATE INDEX IF NOT EXISTS moz_logins_guid_index ON moz_logins (guid)";
+      query =
+        "CREATE INDEX IF NOT EXISTS moz_logins_guid_index ON moz_logins (guid)";
       this._dbConnection.executeSimpleSQL(query);
     }
 
@@ -952,8 +997,9 @@ LoginManagerStorage_mozStorage.prototype = {
     let stmt;
     try {
       stmt = this._dbCreateStatement(query);
-      while (stmt.executeStep())
+      while (stmt.executeStep()) {
         ids.push(stmt.row.id);
+      }
     } catch (e) {
       this.log("Failed getting IDs: " + e);
       throw e;
@@ -968,7 +1014,7 @@ LoginManagerStorage_mozStorage.prototype = {
     for (let id of ids) {
       let params = {
         id,
-        guid: this._uuidService.generateUUID().toString()
+        guid: this._uuidService.generateUUID().toString(),
       };
 
       try {
@@ -985,7 +1031,6 @@ LoginManagerStorage_mozStorage.prototype = {
     }
   },
 
-
   /**
    * Version 3 adds a encType column.
    */
@@ -996,26 +1041,31 @@ LoginManagerStorage_mozStorage.prototype = {
       query = "ALTER TABLE moz_logins ADD COLUMN encType INTEGER";
       this._dbConnection.executeSimpleSQL(query);
 
-      query = "CREATE INDEX IF NOT EXISTS " +
-                  "moz_logins_encType_index ON moz_logins (encType)";
+      query =
+        "CREATE INDEX IF NOT EXISTS " +
+        "moz_logins_encType_index ON moz_logins (encType)";
       this._dbConnection.executeSimpleSQL(query);
     }
 
     // Get a list of existing logins
     let logins = [];
     let stmt;
-    query = "SELECT id, encryptedUsername, encryptedPassword " +
-                "FROM moz_logins WHERE encType isnull";
+    query =
+      "SELECT id, encryptedUsername, encryptedPassword " +
+      "FROM moz_logins WHERE encType isnull";
     try {
       stmt = this._dbCreateStatement(query);
       while (stmt.executeStep()) {
         let params = { id: stmt.row.id };
         // We will tag base64 logins correctly, but no longer support their use.
-        if (stmt.row.encryptedUsername.charAt(0) == "~" ||
-            stmt.row.encryptedPassword.charAt(0) == "~")
+        if (
+          stmt.row.encryptedUsername.charAt(0) == "~" ||
+          stmt.row.encryptedPassword.charAt(0) == "~"
+        ) {
           params.encType = Ci.nsILoginManagerCrypto.ENCTYPE_BASE64;
-        else
+        } else {
           params.encType = Ci.nsILoginManagerCrypto.ENCTYPE_SDR;
+        }
         logins.push(params);
       }
     } catch (e) {
@@ -1044,7 +1094,6 @@ LoginManagerStorage_mozStorage.prototype = {
     }
   },
 
-
   /**
    * Version 4 adds timeCreated, timeLastUsed, timePasswordChanged,
    * and timesUsed columns
@@ -1052,7 +1101,12 @@ LoginManagerStorage_mozStorage.prototype = {
   _dbMigrateToVersion4() {
     let query;
     // Add the new columns, if needed.
-    for (let column of ["timeCreated", "timeLastUsed", "timePasswordChanged", "timesUsed"]) {
+    for (let column of [
+      "timeCreated",
+      "timeLastUsed",
+      "timePasswordChanged",
+      "timesUsed",
+    ]) {
       if (!this._dbColumnExists(column)) {
         query = "ALTER TABLE moz_logins ADD COLUMN " + column + " INTEGER";
         this._dbConnection.executeSimpleSQL(query);
@@ -1062,12 +1116,14 @@ LoginManagerStorage_mozStorage.prototype = {
     // Get a list of IDs for existing logins.
     let ids = [];
     let stmt;
-    query = "SELECT id FROM moz_logins WHERE timeCreated isnull OR " +
-            "timeLastUsed isnull OR timePasswordChanged isnull OR timesUsed isnull";
+    query =
+      "SELECT id FROM moz_logins WHERE timeCreated isnull OR " +
+      "timeLastUsed isnull OR timePasswordChanged isnull OR timesUsed isnull";
     try {
       stmt = this._dbCreateStatement(query);
-      while (stmt.executeStep())
+      while (stmt.executeStep()) {
         ids.push(stmt.row.id);
+      }
     } catch (e) {
       this.log("Failed getting IDs: " + e);
       throw e;
@@ -1078,11 +1134,12 @@ LoginManagerStorage_mozStorage.prototype = {
     }
 
     // Initialize logins with current time.
-    query = "UPDATE moz_logins SET timeCreated = :initTime, timeLastUsed = :initTime, " +
-            "timePasswordChanged = :initTime, timesUsed = 1 WHERE id = :id";
+    query =
+      "UPDATE moz_logins SET timeCreated = :initTime, timeLastUsed = :initTime, " +
+      "timePasswordChanged = :initTime, timesUsed = 1 WHERE id = :id";
     let params = {
-      id:       null,
-      initTime: Date.now()
+      id: null,
+      initTime: Date.now(),
     };
     for (let id of ids) {
       params.id = id;
@@ -1100,13 +1157,15 @@ LoginManagerStorage_mozStorage.prototype = {
     }
   },
 
-
   /**
    * Version 5 adds the moz_deleted_logins table
    */
   _dbMigrateToVersion5() {
     if (!this._dbConnection.tableExists("moz_deleted_logins")) {
-      this._dbConnection.createTable("moz_deleted_logins", this._dbSchema.tables.moz_deleted_logins);
+      this._dbConnection.createTable(
+        "moz_deleted_logins",
+        this._dbSchema.tables.moz_deleted_logins
+      );
     }
   },
 
@@ -1129,7 +1188,11 @@ LoginManagerStorage_mozStorage.prototype = {
       for (let host of disabledHosts) {
         try {
           let uri = Services.io.newURI(host);
-          Services.perms.add(uri, PERMISSION_SAVE_LOGINS, Services.perms.DENY_ACTION);
+          Services.perms.add(
+            uri,
+            PERMISSION_SAVE_LOGINS,
+            Services.perms.DENY_ACTION
+          );
         } catch (e) {
           Cu.reportError(e);
         }
@@ -1151,22 +1214,23 @@ LoginManagerStorage_mozStorage.prototype = {
    * are present in the DB we're using.
    */
   _dbAreExpectedColumnsPresent() {
-    let query = "SELECT " +
-                   "id, " +
-                   "hostname, " +
-                   "httpRealm, " +
-                   "formSubmitURL, " +
-                   "usernameField, " +
-                   "passwordField, " +
-                   "encryptedUsername, " +
-                   "encryptedPassword, " +
-                   "guid, " +
-                   "encType, " +
-                   "timeCreated, " +
-                   "timeLastUsed, " +
-                   "timePasswordChanged, " +
-                   "timesUsed " +
-                "FROM moz_logins";
+    let query =
+      "SELECT " +
+      "id, " +
+      "hostname, " +
+      "httpRealm, " +
+      "formSubmitURL, " +
+      "usernameField, " +
+      "passwordField, " +
+      "encryptedUsername, " +
+      "encryptedPassword, " +
+      "guid, " +
+      "encType, " +
+      "timeCreated, " +
+      "timeLastUsed, " +
+      "timePasswordChanged, " +
+      "timesUsed " +
+      "FROM moz_logins";
     try {
       let stmt = this._dbConnection.createStatement(query);
       // (no need to execute statement, if it compiled we're good)
@@ -1175,10 +1239,7 @@ LoginManagerStorage_mozStorage.prototype = {
       return false;
     }
 
-    query = "SELECT " +
-               "id, " +
-               "hostname " +
-            "FROM moz_disabledHosts";
+    query = "SELECT id, hostname FROM moz_disabledHosts";
     try {
       let stmt = this._dbConnection.createStatement(query);
       // (no need to execute statement, if it compiled we're good)
@@ -1191,7 +1252,6 @@ LoginManagerStorage_mozStorage.prototype = {
     return true;
   },
 
-
   /**
    * Checks to see if the named column already exists.
    */
@@ -1241,14 +1301,16 @@ LoginManagerStorage_mozStorage.prototype = {
 
     this._dbClose();
     this._signonsFile.remove(false);
-  }
-
+  },
 }; // end of nsLoginManagerStorage_mozStorage implementation
 
-XPCOMUtils.defineLazyGetter(this.LoginManagerStorage_mozStorage.prototype, "log", () => {
-  let logger = LoginHelper.createLogger("Login storage");
-  return logger.log.bind(logger);
-});
+XPCOMUtils.defineLazyGetter(
+  this.LoginManagerStorage_mozStorage.prototype,
+  "log",
+  () => {
+    let logger = LoginHelper.createLogger("Login storage");
+    return logger.log.bind(logger);
+  }
+);
 
-var component = [LoginManagerStorage_mozStorage];
-this.NSGetFactory = XPCOMUtils.generateNSGetFactory(component);
+var EXPORTED_SYMBOLS = ["LoginManagerStorage_mozStorage"];
